@@ -1,5 +1,7 @@
 package BioUtil::Util;
 
+use File::Path qw/remove_tree/;
+
 require Exporter;
 @ISA    = (Exporter);
 @EXPORT = qw(
@@ -13,6 +15,15 @@ require Exporter;
 
     read_json_file
     write_json_file
+
+    run
+
+    check_positive_integer
+    
+    filename_prefix
+    check_all_files_exist
+    check_in_out_dir 
+    rm_and_mkdir
 );
 
 use vars qw($VERSION);
@@ -21,6 +32,7 @@ use 5.010_000;
 use strict;
 use warnings FATAL => 'all';
 
+use Encode qw/ encode_utf8 /;
 use File::Path qw(make_path remove_tree);
 use File::Find;
 use File::Basename;
@@ -38,16 +50,16 @@ hoping it would be helpful.
 
 =head1 VERSION
 
-Version 2014.0815
+Version 2014.1115
 
 =cut
 
-our $VERSION = 2014.0815;
+our $VERSION = 2014.1115;
 
 =head1 EXPORT
 
     get_file_list
-    
+
     extract_parameters_from_string
     get_parameters_from_file
 
@@ -57,6 +69,15 @@ our $VERSION = 2014.0815;
     read_json_file
     write_json_file
 
+    run
+
+    check_positive_integer
+    
+    filename_prefix
+    check_all_files_exist
+    check_in_out_dir 
+    rm_and_mkdir
+
 =head1 SYNOPSIS
 
   use BioUtil::Util;
@@ -103,7 +124,7 @@ sub get_file_list {
         warn "depth should be positive integer\n";
         return [];
     }
-    print "$dir\n";
+    # print "$dir\n";
     my $depth0 = $dir =~ tr/\//\//;
 
     my $files  = [];
@@ -251,11 +272,12 @@ Example:
 =cut
 
 sub get_column_data {
-    my ( $file, $column ) = @_;
+    my ( $file, $column, $delimiter ) = @_;
     unless ( $column =~ /^(\d+)$/ and $column > 0 ) {
         warn "column number ($column) should be positive integer\n";
         $column = 1;
     }
+    $delimiter = "\t" unless defined $delimiter;
 
     open IN, "<", $file or die "failed to open file: $file\n";
     my @linedata = ();
@@ -263,13 +285,14 @@ sub get_column_data {
     my $n        = 0;
     while (<IN>) {
         s/\r?\n//;
-        @linedata = split /\t/, $_;
+        next if /^\s*#/;
+        @linedata = split /$delimiter/, $_;
         $n = scalar @linedata;
         next unless $n > 0;
 
         if ( $column > $n ) {
             die
-                "number of columns of this line ($n) is less than given column number ($column)\n";
+                "number of columns of this line ($n) is less than given column number ($column)\n$_";
         }
 
         push @data, $linedata[ $column - 1 ];
@@ -291,7 +314,7 @@ Example:
 
 sub read_json_file {
     my ($file) = @_;
-    open IN, "<", $file
+    open IN, "<:encoding(utf8)", $file
         or die "fail to open json file: $file\n";
     my $text;
     while (<IN>) {
@@ -318,10 +341,135 @@ sub write_json_file {
     my ( $hash, $file ) = @_;
     my $json = JSON->new->allow_nonref;
     my $text = $json->pretty->encode($hash);
-    open OUT, ">", $file
+    $text = encode_utf8( $text );
+    open OUT, ">:encoding(utf8)", $file
         or die "fail to open json file: $file\n";
     print OUT $text;
     close OUT;
 }
 
+=head2 run
+
+Run a command
+
+Example:
+    
+    run('date');
+
+=cut
+
+sub run {
+    my ($cmd) = @_;
+    system($cmd);
+
+    if ( $? == -1 ) {
+        die "[ERROR] fail to run: $cmd. Command ("
+            . ( split /\s+/, $cmd )[0]
+            . ") not found\n";
+    }
+    elsif ( $? & 127 ) {
+        printf "[ERROR] command died with signal %d, %s coredump\n",
+            ( $? & 127 ), ( $? & 128 ) ? 'with' : 'without';
+    }
+    else {
+        # 0, ok
+    }
+}
+
+=head2 check_positive_integer
+
+Check Positive Integer
+
+Example:
+    
+    rcheck_positive_integer(1);
+
+=cut
+
+sub check_positive_integer {
+    my ($n) = @_;
+    die "positive integer needed ($n given)"
+        unless $n =~ /^\d+$/ and $n != 0;
+}
+
+=head2 check_positive_integer
+
+Get filename prefix
+
+Example:
+    
+    filename_prefix("test.fa"); # "test"
+    filename_prefix("tmp");     # "tmp"
+
+=cut
+sub filename_prefix {
+    my ($file) = @_;
+    if ( $file =~ /(.+)\..+?$/ ) {
+        return $1;
+    }
+    else {
+        return $file;
+    }
+}
+
+
+=head2 check_all_files_exist
+
+    Check whether all files existed.
+
+=cut
+sub check_all_files_exist {
+    my $flag = 1;
+    for (@_) {
+        if ( not -e $_) {
+            return 0;
+        }
+    }
+    return 1;
+}
+
+=head2 check_in_out_dir
+
+    Check in and out directory.
+
+Example:
+    
+    check_in_out_dir("~/dir", "~/dir.out");
+
+=cut
+
+sub check_in_out_dir {
+    my ( $in, $out ) = @_;
+    die "dir $in not found."
+        unless -e $in;
+
+    die "$in is not a directory.\n"
+        unless -d $in;
+
+    $in =~ s/\/$//;
+    $out =~ s/\/$//;
+    die "out dir shoud be different from in dir!\n"
+        if $in eq $out;
+}
+
+
+=head2 rm_and_mkdir
+
+    Make a directory, remove it firstly if it exists.
+
+Example:
+    
+    rm_and_mkdir("out")
+
+=cut
+
+
+sub rm_and_mkdir {
+    my ($dir) = @_;
+    if ( -e $dir ) {
+        remove_tree($dir) or die "fail to remove: $dir\n";
+    }
+    mkdir $dir or die "fail to mkdir: $dir\n";
+}
+
 1;
